# Copyright (C) 2009 Jamie Nicol
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import rb
import re

class YouTubeResult:
	def __init__(self, yt_entry, on_ready_cb):
		# Store the function. This will be called when we are ready to be added to the db.
		self.on_ready_cb = on_ready_cb
		
		# Fill out the easy to obtain information
		self.title = yt_entry.media.title.text
		self.duration = int(yt_entry.media.duration.seconds)
		self.id = yt_entry.media.player.url.split("=").pop()
		
		# To get just the video from youtube (not the swf player, or a "watch" page), we need to calculate the "t param".
		# This can be done by parsing the watch page. To avoid blocking we'll do this asynchronously
		loader = rb.Loader()
		loader.get_url(yt_entry.media.player.url, self.on_watch_page_loaded)
	
	def on_watch_page_loaded(self, data):
		
		# Use regular expressions to parse the page and find the t param.
		# Thanks to the totem youtube plugin for this! :D
		regexp1 = re.compile ("swfArgs.*\"t\": \"([^\"]+)\"")
		if data != "":
			matches = regexp1.search(data)
			if (matches != None):
				self.t_param = matches.group(1)
		
		# Now we can fill in the video's uri.
		self.uri = "http://www.youtube.com/get_video?video_id=" + self.id + "&t=" + self.t_param
		
		# For the time being this is all we are finding out, so call signal to let the search know we're done.
		# One day hopefully we'll have thumbnails and other awesome stuff.
		self.on_ready_cb(self)
		
		
				
		

