# Copyright (C) 2009 Jamie Nicol
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import rhythmdb
import gdata.youtube.service
import re
from YouTubeResult import YouTubeResult

class YouTubeSearch:
	def __init__(self, search_term, db, entry_type):
		self.search_term = search_term
		self.db = db
		self.entry_type = entry_type
		self.query_model = rhythmdb.QueryModel()
		self.result_count = 0
		self.ready_result_count = 0
		
	# Returns true if the search is complete, false if it isn't
	def is_complete(self):
		return self.ready_result_count == self.result_count
	
	# Called by a YouTubeResult when it is ready.
	def on_result_ready(self, result):
		# Create the db entry and add it to the query model for this search
		entry = self.db.entry_lookup_by_location(result.uri)
		if entry == None:
			entry = self.db.entry_new(self.entry_type, result.uri)
			self.db.set(entry, rhythmdb.PROP_TITLE, result.title)
			self.db.set(entry, rhythmdb.PROP_DURATION, result.duration)
			
		self.query_model.add_entry(entry, -1)
		
		self.ready_result_count += 1

	# Starts searching
	def start(self):
		# Search youtube for videos matching the query.
		# This doesn't take very much time so shouldn't block.
		# It's getting other data about the videos (eg the "t param") which takes a long time.
		service = gdata.youtube.service.YouTubeService()
		query = gdata.youtube.service.YouTubeVideoQuery()
		query.vq = self.search_term
		query.orderby = "relevance"
		query.racy = "include"
		feed = service.YouTubeQuery(query)
		
		# Create a YouTubeResult object for each result.
		# It will take care of getting all other needed information without blocking.
		for entry in feed.entry:
			self.result_count += 1
			result = YouTubeResult(entry, self.on_result_ready)
