# Copyright (C) 2009 Jamie Nicol
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import rb, rhythmdb
import gobject, gtk
from YouTubeSearch import YouTubeSearch

class YouTubeSource(rb.Source):
	def __init__(self):
		rb.Source.__init__(self)
		self.initialised = False
	
	def initialise(self):
		shell = self.props.shell
		
		self.entry_view = rb.EntryView(shell.props.db, shell.get_player(), "", True, False)
		
		query_model = rhythmdb.QueryModel()
		self.props.query_model = query_model
		self.entry_view.set_model(query_model)
		self.entry_view.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.entry_view.set_shadow_type(gtk.SHADOW_IN)
		self.entry_view.append_column(rb.ENTRY_VIEW_COL_TITLE, True)
		self.entry_view.append_column(rb.ENTRY_VIEW_COL_DURATION, True)
		self.entry_view.set_columns_clickable(False)
		
		# Set up the search UI. This could almost definately be done in a more proper way.
		search_entry = gtk.Entry()
		search_button = gtk.Button("Search")
		hbox = gtk.HBox()
		hbox.pack_start(search_entry)
		hbox.pack_start(search_button)
		hbox.set_child_packing(search_entry, False, True, 0, gtk.PACK_START)
		hbox.set_child_packing(search_button, False, True, 2, gtk.PACK_START)
		vbox = gtk.VBox()
		vbox.pack_start(hbox)
		vbox.set_child_packing(hbox, False, False, 2, gtk.PACK_START)
		vbox.pack_start(self.entry_view)
		self.add(vbox)
		self.show_all()
		
		search_button.connect("clicked", self.on_search_button_clicked, search_entry)
		
		# The button isn't behaving as I want it to. Dunno what I'm doing wrong.
		search_entry.set_activates_default(True)
		search_button.set_flags(gtk.CAN_DEFAULT)
		search_button.grab_default()
		
		self.searches = {} # Dictionary of searches, with the search term as keys
		self.current_search = "" # The search term of the search results currently being shown
		
		self.initialised = True
		
	def do_impl_get_entry_view(self):
		return self.entry_view
	
	def do_impl_activate(self):
		if not self.initialised:
			self.initialise()
			
	def do_impl_get_status(self):
		if self.current_search:
			if self.searches[self.current_search].is_complete():
				return (self.props.query_model.compute_status_normal("Found %d result", "Found %d results"), "", 1)
			else:
				return ("Searching for \"" + self.current_search + "\"", "", -1)
			
		else:
			return ("", "", 1)
			
	def do_impl_delete_thyself(self):
		if self.initialised:
			self.props.shell.props.db.entry_delete_by_type(self.props.entry_type)
		rb.Source.do_impl_delete_thyself(self)
		
	def do_impl_can_add_to_queue(self):
		return True
		
	def on_search_button_clicked(self, button, entry):
		# Only do anything if there is text in the search entry
		if entry.get_text():
			# If this term has never been searched before then create a new search object, add it to the dictionary, and start the search. 
			if not entry.get_text() in self.searches:
				self.searches[entry.get_text()] = YouTubeSearch(entry.get_text(), self.props.shell.props.db, self.props.entry_type)
				self.searches[entry.get_text()].start()
			
			# Update the entry view and source so the display the query model relevant to the current search
			self.current_search = entry.get_text()
			self.props.query_model = self.searches[self.current_search].query_model
			self.entry_view.set_model(self.props.query_model)
			
			
gobject.type_register(YouTubeSource)
